function ReaderOutput = ReadFFSData(ReaderInput)

% Readers are .m functions that look at data files and produce standard matlab data

% ReaderInput
%     .Files - cell array of file names passed to the reader
%     .HelperFunctions - cell array of helper function names to be used with EnableHelperFunction
%     .ProgramStatusIndicator - a handle to the Program Status Indicator on the front panel so that status can be updated from within the reader

% ReaderOutput
%     .Data - a struct (of arbitrary structure) that you want written to the trial file
%     .ReaderName - a string naming the data so that the trial file will have Trial.ReaderName = Data
%     .TrialName - a string that either names the trial or is empty.  The reader producing a trial name must be run first
%     .TrialDescription - a string describing the trial so that Trial.TrialDescription = TrialDescription

% parameters
BufferLength = 100;
ActiveMotorInd = [1:7]';
ViconMarkerNames = {};
ExportVarNames = {};
NumMotorsTotal = 20;
ActiveMotorLabel = {'LUM';'FPI';'FDS';'FDP';'FDI';'EIP';'EDC'};
SampFreq = 100;

% helper functions
strsep = EnableHelperFunction([],'strsep.m');

% get data
set(ReaderInput.ProgramStatusIndicator,'string','Loading data file');
pause(0.1);
dm = load(ReaderInput.Files{1});
set(ReaderInput.ProgramStatusIndicator,'string','Finished loading data file');
pause(0.1);

A = reshape(dm',size(dm,1)*size(dm,2),1);
NumDataColumns = size(dm,2)/BufferLength;
B = reshape(A,NumDataColumns,length(A)/NumDataColumns)';

% format data appropriately
ReaderOutput.ReaderName = 'Labview';
temp = strsep(ReaderInput.Files{1},ReaderInput.PathSep);
ReaderOutput.TrialName = temp{end}(1:end-4);
ReaderOutput.TrialDescription = 'this is a test';

ReaderOutput.Data.SampFreq = SampFreq;
ReaderOutput.Data.Time = B(:,1);

for i = 1:length(ActiveMotorInd)
    ReaderOutput.Data.Motor(i).Name = ActiveMotorLabel{i};
    ReaderOutput.Data.Motor(i).Length = B(:,1+1+ActiveMotorInd(i));
    ReaderOutput.Data.Motor(i).Force = B(:,1+NumMotorsTotal+1+ActiveMotorInd(i));
    ReaderOutput.Data.Motor(i).Voltage = B(:,1+2*NumMotorsTotal+1+ActiveMotorInd(i));
end

ReaderOutput.Data.FingertipWrench = B(:,1+3*NumMotorsTotal+1:1+3*NumMotorsTotal+1+5);


NumViconMarkers = length(ViconMarkerNames);
for i = 1:length(ViconMarkerNames)
    ReaderOutput.Data.Marker(i).Name = ViconMarkerNames{i};
    L = B(:,1+3*NumMotorsTotal+6+1+3*(i-1):1+3*NumMotorsTotal+6+1+3*(i-1)+2);
    L(L==1000000) = NaN;
    ReaderOutput.Data.Marker(i).Location = L;
end

for i = 1:length(ExportVarNames)
    ReaderOutput.Data.ExportVar(i).Name = ExportVarNames{i};
    L = B(:,1+3*NumMotorsTotal+6+3*NumViconMarkers+i);
    ReaderOutput.Data.ExportVar(i).Data = L;
end

set(ReaderInput.ProgramStatusIndicator,'string','Finished processing, making a copy of the data file');
pause(0.1);