function ReaderOutput = ReadAllFiles(ReaderInput)

% Readers are .m functions that look at data files and produce standard matlab data

% ReaderInput
%     .Files - cell array of file names passed to the reader
%     .HelperFunctions - cell array of helper function names to be used with EnableHelperFunction
%     .ProgramStatusIndicator - a handle to the Program Status Indicator on the front panel so that status can be updated from within the reader

% ReaderOutput
%     .Data - fields of Data will become fields of Trial
%     .ReaderName - a string naming the data so that the trial file will have Trial.ReaderName = Data
%     .TrialName - a string that either names the trial or is empty.  The reader producing a trial name must be run first
%     .TrialDescription - a string describing the trial so that Trial.TrialDescription = TrialDescription

strsep = EnableHelperFunction(ReaderInput.HelperFunctions,'strsep.m');
ProgramStatusIndicator = ReaderInput.ProgramStatusIndicator;

set(ProgramStatusIndicator,'string','Starting ''ReadAllFiles'' reader...');
pause(0.1);

ReaderOutput.ReaderName = 'AllData';

InputFile = ReaderInput.Files{1};
OutputFile = ReaderInput.Files{2};

iFile = strsep(InputFile,ReaderInput.PathSep);
oFile = strsep(OutputFile,ReaderInput.PathSep);

ReaderOutput.Data.Labview.InputFile = iFile{end};
ReaderOutput.Data.Labview.OutputFile = oFile{end};

Input = load(InputFile);
MaxLines = size(Input,1);

fid = fopen(OutputFile);
% the first line is always the trial name
Line1 = fgetl(fid);
ReaderOutput.TrialName = Line1;
ReaderOutput.TrialName
% the second line is always the trial description
Line2 = fgetl(fid);
ReaderOutput.TrialDescription = Line2;

% the third line is always the trial date
Line3 = fgetl(fid);
ReaderOutput.Data.Labview.Date = Line3;

% the fourth line can be the time or the input mode
Line4 = fgetl(fid);
switch isempty(strfind(Line4,'Mode'))
    case 0 % the file type is two time marking
        Type = strsep(Line4,',');
        ReaderOutput.Data.Labview.InputMode = Type{2};
    case 1 % thie file is single time marking
        ReaderOutput.Data.Labview.Time = Line4;
end

% line5 can be the mode or frequency information
Line5 = fgetl(fid);
switch isempty(strfind(Line5,'Mode'))
    case 0 % line 5 has the mode info
        Type = strsep(Line5,',');
        ReaderOutput.Data.Labview.InputMode = Type{2};
    case 1 % line 5 has the frequency info
        Freq = strsep(Line5,',');
end

% line 6 can be either frequency info or muscle names
Line6 = fgetl(fid);
switch isempty(strfind(Line6,'Hz'))
    case 0 % line 6 is frequency info
        Freq = strsep(Line6,',');
    case 1 % line 6 is muscle names
        MuslNames = strsep(Line6,','); 
end

%line 7 is either muscle names or column labels
Line7 = fgetl(fid);
switch isempty(strfind(Line7,'Index'))
    case 0 % line 7 is column labels
        Labels = strsep(Line7,',')';
    case 1 % line 7 is muscle names
        MuslNames = strsep(Line7,',');
end

% line 8 is either column labels or start time
Line8 = fgetl(fid);
switch isempty(strfind(Line8,'Index'))
    case 0 % line 8 is column labels
        Labels = strsep(Line8,',')';
        TwoTimeMarkFlag = 0;
    case 1 % line 8 is start time
        ReaderOutput.Data.Labview.DataStartTime = Line8;
        TwoTimeMarkFlag = 1;
end

fclose(fid);

LenLabels = length(Labels);

NumLines = str2num(Freq{1});

set(ReaderInput.ProgramStatusIndicator,'string','Reading Labview Output File ...');
pause(0.5);
Output = csvread(OutputFile,8,0,[8 0 8+NumLines-1 LenLabels-1]);
set(ReaderInput.ProgramStatusIndicator,'string','Reading Labview Output File ... Finished');
pause(1);

ReaderOutput.Data.Labview.NumLines = NumLines;
ReaderOutput.Data.Labview.SampFreq = str2num(Freq{2});
ReaderOutput.Data.Labview.SampFreqUnits = Freq{3};

if TwoTimeMarkFlag == 1
    fid = fopen(OutputFile);
    fseek(fid,-10,'eof');
    ReaderOutput.Data.Labview.DataEndTime = fgetl(fid);
    fclose(fid);
end
    
NumMusls = (length(Labels)-8)/2;

ReaderOutput.Data.Labview.Index = Output(:,1);
ReaderOutput.Data.Labview.SyncSignal = Output(:,2);

for Musl = 1:NumMusls
    ReaderOutput.Data.Labview.Motor(Musl,1).Name = MuslNames{Musl};
    ReaderOutput.Data.Labview.Motor(Musl,1).Input = Input(:,1+Musl);
    if strcmp(ReaderOutput.Data.Labview.InputMode,'Spring') == 1 && length(Type) > 2
        ReaderOutput.Data.Labview.Motor(Musl,1).SpringConstant = str2num(Type{3+Musl});
        ReaderOutput.Data.Labview.Motor(Musl,1).SpringConstantUnits = Type{3};
    end
    ReaderOutput.Data.Labview.Motor(Musl,1).Length = Output(:,2+Musl);
    ReaderOutput.Data.Labview.Motor(Musl,1).Force = Output(:,2+NumMusls+Musl);
end

ReaderOutput.Data.Labview.FingertipLoadCell.Names = Labels(2+2*NumMusls+1:2+2*NumMusls+1+5);
size(Output)
ReaderOutput.Data.Labview.FingertipLoadCell.Voltage = Output(:,2+2*NumMusls+1:end); 

%--------
%read vicon

ViconDataFile = ReaderInput.Files{3};

vFile = strsep(ViconDataFile,ReaderInput.PathSep);

fid = fopen(ViconDataFile);

Title = strsep(fgetl(fid),',');
Date = strsep(fgetl(fid),',');

fname = strsep(ViconDataFile,'.');

ReaderOutput.Data.Vicon.File = vFile{end};

ReaderOutput.Data.Vicon.Date = Date{2};

Time = strsep(fgetl(fid),',');

ReaderOutput.Data.Vicon.Time = Time{2};

Type = strsep(fgetl(fid),',');

ReaderOutput.Data.Vicon.Type = Type{2};

Description = strsep(fgetl(fid),',');

ReaderOutput.Data.Vicon.Description = Description{2};
ReaderOutput.Data.Vicon.Description

Notes = strsep(fgetl(fid),',');

ReaderOutput.Data.Vicon.Notes = Notes{2};

fgetl(fid);   % skipped line
fgetl(fid);   % TRAJECTORIES

SampFreq = strsep(fgetl(fid),',');

ReaderOutput.Data.Vicon.Motion.SampFreq = str2num(SampFreq{1});
ReaderOutput.Data.Vicon.Motion.SampFreqUnits = SampFreq{2};

Objects = strsep(fgetl(fid),',');

c = 1;
for i = 1:length(Objects)
    if isempty(Objects{i}) == 0
        Markers{c} = Objects{i};
        c = c+1;
    end
end    
    
Labels = strsep(fgetl(fid),',');

fclose(fid);

fid = fopen(ViconDataFile);

% determine exact number of lines in vicon file
fseek(fid,-1000,'eof');
Line = fgetl(fid);
while Line ~= -1
    temp = strsep(Line,',');
    NumViconLines = str2num(temp{1});
    Line = fgetl(fid);
end

fclose(fid);

set(ReaderInput.ProgramStatusIndicator,'string','Reading Vicon File ...');
pause(0.5)
Motion = csvread(ViconDataFile,11,0,[11 0 11+NumViconLines-1 length(Labels)-1]);
set(ReaderInput.ProgramStatusIndicator,'string','Reading Vicon File ... Finished');
pause(1);

ReaderOutput.Data.Vicon.Motion.Index = Motion(:,1);
for i = 1:length(Markers)
   ReaderOutput.Data.Vicon.Motion.Marker(i,1).Name = Markers{i};
   Start = 2+3*(i-1);
   Finish = Start+2;
   ReaderOutput.Data.Vicon.Motion.Marker(i,1).Data = Motion(:,Start:Finish);
end

set(ProgramStatusIndicator,'string','''ReadAllFiles'' reader finished');
pause(0.1);