function ReaderOutput = ReadLabviewDataFiles(ReaderInput)

% Readers are .m functions that look at data files and produce standard matlab data

% ReaderInput
%     .Files - cell array of file names passed to the reader
%     .HelperFunctions - cell array of helper function names to be used with EnableHelperFunction
%     .ProgramStatusIndicator - a handle to the Program Status Indicator on the front panel so that status can be updated from within the reader

% ReaderOutput
%     .Data - a struct (of arbitrary structure) that you want written to the trial file
%     .ReaderName - a string naming the data so that the trial file will have Trial.ReaderName = Data
%     .TrialName - a string that either names the trial or is empty.  The reader producing a trial name must be run first
%     .TrialDescription - a string describing the trial so that
%     Trial.TrialDescription = TrialDescription

InputFile = ReaderInput.Files{1};
OutputFile = ReaderInput.Files{2};

ReaderOutput.Data.InputFile = InputFile;
ReaderOutput.Data.OutputFile = OutputFile;

strsep = EnableHelperFunction(ReaderInput.HelperFunctions,'strsep.m');

ProgramStatusIndicator = ReaderInput.ProgramStatusIndicator;

Input = load(InputFile);
MaxLines = size(Input,1);

fid = fopen(OutputFile);
% the first line is always the trial name
Line1 = fgetl(fid);
ReaderOutput.TrialName = Line1;
% the second line is always the trial description
Line2 = fgetl(fid);
ReaderOutput.TrialDescription = Line2;

% the third line is always the trial date
Line3 = fgetl(fid);
ReaderOutput.Data.Date = Line3;

% the fourth line can be the time or the input mode
Line4 = fgetl(fid);
switch isempty(strfind(Line4,'Mode'))
    case 0 % the file type is two time marking
        Type = strsep(Line4,',');
        ReaderOutput.Data.InputMode = Type{2};
    case 1 % thie file is single time marking
        ReaderOutput.Data.Time = Line4;
end

% line5 can be the mode or frequency information
Line5 = fgetl(fid);
switch isempty(strfind(Line5,'Mode'))
    case 0 % line 5 has the mode info
        Type = strsep(Line5,',');
        ReaderOutput.Data.InputMode = Type{2};
    case 1 % line 5 has the frequency info
        Freq = strsep(Line5,',');
end

% line 6 can be either frequency info or muscle names
Line6 = fgetl(fid);
switch isempty(strfind(Line6,'Hz'))
    case 0 % line 6 is frequency info
        Freq = strsep(Line6,',');
    case 1 % line 6 is muscle names
        MuslNames = strsep(Line6,','); 
end

%line 7 is either muscle names or column labels
Line7 = fgetl(fid);
switch isempty(strfind(Line7,'Index'))
    case 0 % line 7 is column labels
        Labels = strsep(Line7,',')';
    case 1 % line 7 is muscle names
        MuslNames = strsep(Line7,',');
end

% line 8 is either column labels or start time
Line8 = fgetl(fid);
switch isempty(strfind(Line8,'Index'))
    case 0 % line 8 is column labels
        Labels = strsep(Line8,',')';
        TwoTimeMarkFlag = 0;
    case 1 % line 8 is start time
        ReaderOutput.Data.DataStartTime = Line8;
        TwoTimeMarkFlag = 1;
end

LenLabels = length(Labels);

NumLines = str2num(Freq{1});

Output = zeros(NumLines,LenLabels);

Line = fgetl(fid);
vec = strsep(Line,',');
LineNum = 0;
while length(vec) == LenLabels
    statusUpdate = sprintf('Processing, %.0f %% complete',100*LineNum/NumLines); 
    set(ProgramStatusIndicator,'string',statusUpdate);
    pause(0.00001)
    LineNum = LineNum+1;
    for j = 1:length(vec)
        temp = str2num(vec{j});
        if isempty(temp) == 0
            Output(LineNum,j) = temp;
        else
            Output(LineNum,j) = NaN;
        end
    end
    Line = fgetl(fid);
    vec = strsep(Line,',');
end

ActualLines = LineNum;

NumStoredLines = min(MaxLines,ActualLines)

Output = Output(1:NumStoredLines,:);

ReaderOutput.Data.NumLines = NumStoredLines;
ReaderOutput.Data.SamplingFreq = str2num(Freq{2});
ReaderOutput.Data.SamplingFreqUnits = Freq{3};

if TwoTimeMarkFlag == 1
    ReaderOutput.Data.DataEndTime = Line;
end
    
%NumMusls = (length(Labels)-8)/2;
NumMusls = 7;

size(Output)

ReaderOutput.Data.Index = Output(:,1);
ReaderOutput.Data.SyncSignal = Output(:,2);

for Musl = 1:NumMusls
    ReaderOutput.Data.Motor(Musl,1).Name = MuslNames{Musl};
    ReaderOutput.Data.Motor(Musl,1).Input = Input(1:NumStoredLines,1+Musl);
    if strcmp(ReaderOutput.Data.InputMode,'Spring') == 1 && length(Type) > 2
        ReaderOutput.Data.Motor(Musl,1).SpringConstant = str2num(Type{3+Musl});
        ReaderOutput.Data.Motor(Musl,1).SpringConstantUnits = Type{3};
    end
    ReaderOutput.Data.Motor(Musl,1).Length = Output(:,2+Musl);
    ReaderOutput.Data.Motor(Musl,1).Force = Output(:,2+NumMusls+Musl);
end

ReaderOutput.Data.FingertipLoadCell.Names = Labels(2+2*NumMusls+1:2+2*NumMusls+1+5);
ReaderOutput.Data.FingertipLoadCell.Voltage = Output(:,2+2*NumMusls+1:end); 

fclose(fid);

ReaderOutput.ReaderName = 'Labview';