function ReaderOutput = CppsFmri(ReaderInput)

% Readers are .m functions that look at data files and produce standard matlab data

% ReaderInput
%     .Files - cell array of file names passed to the reader
%     .HelperFunctions - cell array of helper function names to be used with EnableHelperFunction
%     .ProgramStatusIndicator - a handle to the Program Status Indicator on the front panel so that status can be updated from within the reader

% ReaderOutput
%     .Data - a struct (of arbitrary structure) that you want written to the trial file
%     .ReaderName - a string naming the data so that the trial file will have Trial.ReaderName = Data
%     .TrialName - a string that either names the trial or is empty.  The reader producing a trial name must be run first
%     .TrialDescription - a string describing the trial so that Trial.TrialDescription = TrialDescription
%
% Last updated: 2011-11-23 
%
% List of inputs:
% 1. Physiological data in mat file
% 2. Preprocessed nii imaging file (native)
% 3. Melodic nii imaging file (native)
% 4. Melodic mix file (text file)
% 5. T1-weighted structural scan (nii.gz)

%FslDir = '/usr/local/fsl/bin';

set(ReaderInput.ProgramStatusIndicator,'string','Reader "CppsFmri" started...');pause(1);

% enable helper functions
strsep = EnableHelperFunction([],'strsep.m');

% load physiological data
try
    set(ReaderInput.ProgramStatusIndicator,'string','Loading physiological data');pause(0.5);
    load(ReaderInput.Files{1});
    set(ReaderInput.ProgramStatusIndicator,'string','Physiolgical data loaded.');pause(0.5);
catch
    set(ReaderInput.ProgramStatusIndicator,'string','ERROR: There was an error reading the physiological data.');
    error('Databaser had an error');
end
% data will be available in a struct called 'Trial'
    
% will be reading nii files, so change directory to NifTi helpers  
BaseDirectory = cd;
cd('HelperFunctions/nifti');

% load raw nii file
try 
    set(ReaderInput.ProgramStatusIndicator,'string','Loading preprocessed nii (native coord.)...');pause(0.5);
    system(['gunzip ' ReaderInput.Files{2}]);
    tempfile = ReaderInput.Files{2}(1:end-3);
    nii_pre_native = load_nii(tempfile);
    system(['gzip ' tempfile]);
    set(ReaderInput.ProgramStatusIndicator,'string','Preprocessed nii (native coord.) loaded.');pause(0.5);
catch err
    set(ReaderInput.ProgramStatusIndicator,'string','ERROR: There was an error reading the preprocessed nii (native coord.) file. See matlab command window for details.');
    cd(BaseDirectory);
    error(['Databaser had an error: ' err.message]);
end

% load MELODIC independent components results
try
    set(ReaderInput.ProgramStatusIndicator,'string','reading the preprocessed MELODIC results (native coord.)...');pause(0.5);
    system(['gunzip ' ReaderInput.Files{3}]);
    tempfile = ReaderInput.Files{3}(1:end-3);
    nii_pre_melodic = load_nii(tempfile);
    system(['gzip ' tempfile]);
    set(ReaderInput.ProgramStatusIndicator,'string','Preprocessed MELODIC results (native coord.) loaded.');pause(0.5);
    mix_pre_melodic = dlmread(ReaderInput.Files{4});
catch err
    set(ReaderInput.ProgramStatusIndicator,'string','ERROR: There was an error reading the preprocessed MELODIC results (native coord.). See matlab command window for details.');
    cd(BaseDirectory);
    error(['Databaser had an error: ' err.message]);  
end

% load structural data
try
    set(ReaderInput.ProgramStatusIndicator,'string','reading the T1-weighted structural scan...');pause(0.5);
    system(['gunzip ' ReaderInput.Files{5}]);
    tempfile = ReaderInput.Files{5}(1:end-3);
    nii_structural = load_nii(tempfile);
    system(['gzip ' tempfile]);
    set(ReaderInput.ProgramStatusIndicator,'string','T1-weighted structural scan loaded.');pause(0.5);
    cd(BaseDirectory);
catch err
    set(ReaderInput.ProgramStatusIndicator,'string','ERROR: There was an error reading the T1-weighted structural scan. See matlab command window for details.');
    cd(BaseDirectory);
    error(['Databaser had an error: ' err.message]);
end



% structuring data

set(ReaderInput.ProgramStatusIndicator,'string','Structuring data...');
pause(0.5)

temp = strsep(ReaderInput.Files{1},'_');
temp2 = strsep(temp{1},'/');

ReaderOutput.TrialName = temp2{end};
ReaderOutput.TrialDescription = Run.Description;
ReaderOutput.ReaderName = 'AllData';

% fill in the data
ReaderOutput.Data.ParticipantNumber = Run.ParticipantNumber;
ReaderOutput.Data.ParticipantGroup = Run.ParticipantGroup;
ReaderOutput.Data.SessionNumber = Run.SessionNumber;
ReaderOutput.Data.ScannerID = Run.ScannerID;
ReaderOutput.Data.RunNumber = Run.RunNumber;
ReaderOutput.Data.PhySampFreq = Run.SamplingFrequency;
ReaderOutput.Data.PhyChannel = Run.Channel;
try
    ReaderOutput.Data.Scanner = Run.Scanner;
catch
    ReaderOutput.Data.Scanner = [];
end

try 
    ReaderOutput.Data.Stimulus = Run.Stimulus;
catch
    ReaderOutput.Data.Stimulus = [];
end

TR = 2;
NumImageFrames = size(nii_pre_native.img,4);

ReaderOutput.Data.Imaging.Structural.Header = nii_structural.hdr;
ReaderOutput.Data.Imaging.Structural.Volume = nii_structural.img;
ReaderOutput.Data.Imaging.Preprocessed.Time = TR*[0:NumImageFrames-1]';
ReaderOutput.Data.Imaging.Preprocessed.Frames = nii_pre_native.img;
ReaderOutput.Data.Imaging.Preprocessed.IC.Comp = nii_pre_melodic.img;
ReaderOutput.Data.Imaging.Preprocessed.IC.Weights = mix_pre_melodic;
ReaderOutput.Data.Imaging.Preprocessed.Header = nii_pre_native.hdr;

set(ReaderInput.ProgramStatusIndicator,'string','Reader CppsFmri finished ... copying data files.');
pause(0.5);